/**
 * @file analog_in.h
 * @author your name (you@domain.com)
 * @brief read a value from a connected device giving an analog signal.
 * @version 0.1
 * @date 2022-07-18
 * 
 * @copyright Copyright (c) 2022
 * 
 */

#include <drivers/adc.h>

#include <stdint.h>

struct analog_in_cfg
{
    // static initialisation:
    uint8_t input; // the analog pin
    uint32_t output_ohms; 
    uint32_t full_ohms;

    // dynamic config: initialized with analog_in_init()
    struct adc_channel_cfg adc_channel;
    struct adc_sequence my_sequence;

    uint8_t power_output; // power suply of sensor: TODO use an enum or reference to a device?
    uint16_t sensor_boot_time_ms; // time required by sensor to give a reliable value after applying power

    // results: reset upon analog_in_init(), or set upon a succesfull read.
    float accuracy; 
};

// easy init for: struct analog_in_cfg
#define ANALOG_IN_INST( divider ) \
{\
    .input  = DT_IO_CHANNELS_INPUT(DT_PATH(divider)),\
    .output_ohms = DT_PROP(DT_PATH(divider), output_ohms),\
    .full_ohms = DT_PROP(DT_PATH(divider), full_ohms)\
}

/** Initialize the input for measuring with a specific sensor.
 * The resolution and oversampling will be set to match the required accuracy.
 * Actual resulting accuracy is set in ain->accuracy.
 * Only required when the configuration changes.
 * @return 0 ::= OK, <0 ::= error code
 */
int32_t analog_in_init(struct analog_in_cfg *ain, 
    float v_max, float accuracy,
    uint8_t power_output,
    uint16_t sensor_boot_time_ms);

/** read an analog input.
 * When needed: the sensor power is turned on and off.
 * Watch out: this can take some time due to sensor boot time!
 * @param ain : initialized input
 * @return 0 ::= ok, <0 ::= error code
 */
int32_t analog_in_read(struct analog_in_cfg *ain, float *v_measured);

typedef void (*analog_in_cb)(uint32_t result);

/** read an analog input.
 * When needed: the sensor power is turned on and off.
 * *v_measured is only set after the callback is called with OK
 */
void analog_in_read_async(struct analog_in_cfg *ain, float *v_measured, analog_in_cb *cb);
